/*
 * Copyright (c) 1996-1997, InetSoft Technology Corp, All Rights Reserved.
 *
 * The software and information contained herein are copyrighted and 
 * proprietary to InetSoft Technology Corp. This software is furnished 
 * pursuant to a written license agreement and may be used, copied, 
 * transmitted, and stored only in accordance with the terms of such 
 * license and with the inclusion of the above copyright notice. Please 
 * refer to the file "COPYRIGHT" for further copyright and licensing 
 * information. This software and information or any other copies 
 * thereof may not be provided or otherwise made available to any other 
 * person. 
 */
package itrade.gui;

import java.awt.*;

class BorderCenterLayout extends BorderLayout {
   private Component  center;
   private int        stretch = V_STATIC | H_STATIC;
   
   /*
    * Don't strech vertically.
    */
   public static final int V_STATIC = 1;

   /*
    * Don't strech horizontally.
    */
   public static final int H_STATIC = 2;
   
   BorderCenterLayout() {
      super();
   }
   
   public void addLayoutComponent(Component comp, Object constraints) {
      String pos = ((Constraints)constraints).border;      
      
      if (pos.equals("Center")) {
         stretch = ((Constraints)constraints).stretch;
	 center = comp;
      }
      super.addLayoutComponent(comp, pos);
   }
   
   public void layoutContainer(Container target) {   
      int  newY, newH, newX, newW;
      super.layoutContainer(target);
      
      if (center != null && center.isVisible()) {
         int h = center.getSize().height;
         int y = center.getLocation().y;
         int w = center.getSize().width;
         int x = center.getLocation().x;
	 int preferH = center.getPreferredSize().height;
	 int preferW = center.getPreferredSize().width;

	 if ((stretch & V_STATIC) != 0 && preferH < h) {
	    newY = y + (h - preferH) / 2;
	    newH = preferH;
	 }
	 else {
	    newY = y;
	    newH = h;
	 }
	 if ((stretch & H_STATIC) != 0 && preferW < w) {
	    System.out.println("new x");
	    newX = x + (w - preferW) / 2;
	    newW = preferW;
	 }
	 else {
	    newX = x;
	    newW = w;
	 }
    	   
	 if (newX != x || newY != y || newW != w || newH != h) {
	   center.setBounds(newX, newY, newW, newH);
	 }
      }      
   }
   
   /**
    * Constraints object can be used to define the layout constraints
    * of child components. The default alignment is V_STATIC | H_STATIC.
    */
   public static class Constraints implements java.io.Serializable {
      String  border;
      int     stretch;
      
      /**
       * Construct a layout constraints with specified parameters 
       * @param border border location.
       * @param strech indicate which orientation can be stretched.
       */
      public Constraints(String border, int stretch) {
	 this.border = border; this.stretch = stretch;
      }
   }   
}









