/*
 * Copyright (c) 1996-1997, InetSoft Technology Corp, All Rights Reserved.
 *
 * The software and information contained herein are copyrighted and 
 * proprietary to InetSoft Technology Corp. This software is furnished 
 * pursuant to a written license agreement and may be used, copied, 
 * transmitted, and stored only in accordance with the terms of such 
 * license and with the inclusion of the above copyright notice. Please 
 * refer to the file "COPYRIGHT" for further copyright and licensing 
 * information. This software and information or any other copies 
 * thereof may not be provided or otherwise made available to any other 
 * person. 
 */
package itrade.gui;

import java.awt.*;

class BorderShareLayout extends BorderLayout {
   boolean     eastShare;
   boolean     westShare;
   boolean     centerShare;
   boolean     northShare;
   boolean     southShare;
   Component   east;
   Component   west;
   Component   north;
   Component   south;
   Component   center;
   
   public void addLayoutComponent(Component comp, Object constraints) {
      boolean share = ((Constraints)constraints).share;
      String  name = ((Constraints)constraints).border;
      
      if ("Center".equals(name)) {
	 center = comp;
	 centerShare = share;
      } else if ("North".equals(name)) {
	 northShare = share;
	 north = comp;
      } else if ("South".equals(name)) {
	 southShare = share;
	 south = comp;
      } else if ("East".equals(name)) {
	 eastShare = share;
	 east = comp;
      } else if ("West".equals(name)) {
	 westShare = share;
	 west = comp;
      }
      
      super.addLayoutComponent(comp, name);
   }
   
   public void layoutContainer(Container target) {
      Insets insets = target.getInsets();
      int top = insets.top;
      int bottom = target.getSize().height - insets.bottom;
      int left = insets.left;
      int right = target.getSize().width - insets.right;
	
      int  vFactor = 1, hFactor = 1;
      int  height = 0, width = 0;
      int  avgH = 0, avgW = 0;

      /*
       * step 1: let border layout put components into prefered size.
       */
      super.layoutContainer(target);
      
      
      /*
       * step 2: calculate average size for components which want to share.
       */
      if (center != null) {
	 height = center.getSize().height;
	 width = center.getSize().width;
      }

      if (north != null && northShare) {
	 vFactor++;
	 height += north.getSize().height;
      }
      if (south != null && southShare) {
	 vFactor++;
	 height += south.getSize().height;	 
      }
      if (east != null && eastShare) {
	 hFactor++;
	 width += east.getSize().width;	 	 
      }
      if (west != null && westShare) {
	 hFactor++;
	 width += west.getSize().width;	 	 
      }
      
      if (vFactor > 0) {
         avgH = height / vFactor;
      }
      if (hFactor > 0) {
         avgW = width / hFactor;
      }
      
      /*
       * step 3: change size of components which participate in sharing.
       */
      if (north != null) {
	 if (northShare) {
   	     north.setBounds(left, top, right - left, avgH);
	     top += avgH + getVgap();
	 }
	 else {
	     top += north.getSize().height + getVgap();
	 }
      }
      
      if (south != null) {
	 if (southShare) {
   	    south.setBounds(left, bottom-avgH, right - left, avgH);
	    bottom -= avgH + getVgap();
	 }
         else {
	    bottom -= south.getSize().height + getVgap();
	 }
      }
   
      if (east != null) {
	 if (eastShare) {
            east.setBounds(right - avgW, top, avgW, bottom - top);
	    right -= avgW + getHgap();
	 }
	 else {
	    east.setBounds(right - east.getSize().width, top,
			   east.getSize().width, bottom - top);
	    right -= east.getSize().width + getHgap();
	 }
      }
      if (west != null) {
	 if (westShare) {
            west.setBounds(left, top, avgW, bottom - top);
	    left += avgW + getHgap();
	 }
	 else {
	    west.setBounds(left, top,
			   west.getSize().width, bottom - top);
	    left += west.getSize().width + getHgap();
	 }
      }
      if (center != null) {
	 center.setBounds(left, top, right - left, bottom - top);
      }
   }      
   
   /**
    * Constraints object can be used to define the layout constraints
    * of child components. The default alignment is V_FILL | H_FILL.
    */
   public static class Constraints implements java.io.Serializable {
      String  border;
      boolean share;
      
      /**
       * Construct a layout constraints with specified parameters 
       * @param border border location.
       * @param share indicate that if this component will share extra space.
       */
      public Constraints(String border, boolean share) {
	 this.border = border; this.share = share;
      }
   }   
}

