/*
 * Copyright (c) 1996-1997, InetSoft Technology Corp, All Rights Reserved.
 *
 * The software and information contained herein are copyrighted and 
 * proprietary to InetSoft Technology Corp. This software is furnished 
 * pursuant to a written license agreement and may be used, copied, 
 * transmitted, and stored only in accordance with the terms of such 
 * license and with the inclusion of the above copyright notice. Please 
 * refer to the file "COPYRIGHT" for further copyright and licensing 
 * information. This software and information or any other copies 
 * thereof may not be provided or otherwise made available to any other 
 * person. 
 */
package itrade.gui;

import java.awt.*;
import java.awt.event.*;
import java.rmi.*;
import java.rmi.server.*;
import tea.set.*;
import bamboo.pipe.*;

class Editing {
   /*
    * GUI objects on the Edit panel.
    */
   public Effect3D    editPanelDecor;
   public Panel       editPanel;
   public Panel       editBasePanel;
   public Panel       fieldPanel;
   public Panel       memoPanel;
   public Panel       cmdPanel;
   public Panel       uFieldPanel;
   public Grid        lFieldPanel;
   
   public Label     tradeIDLabel;
   public Label     tradeIDField;
   public Label     buySellLabel;
   public Spinner   buySellField;
   public Label     tickerLabel;
   public TextField tickerField;
   public Label     priceLabel;
   public TextField priceField;
   public Label     quantityLabel;
   public TextField quantityField;
   public Label     trdDateLabel;
   public TextField trdDateField;
   public Label     acctLabel;
   public Label     acctField;
   public Label     memoLabel;
   public TextArea  memoField;
   public Checkbox  newFlagField;
   
   public ImageButton    saveButton;
   public ImageButton    deleteButton;   

   public Producer    dbRqtSend;
   public int         newIdCount = 0;   
   
   Editing(Panel parent, BambooPipe msgServer) {
      String[]  buySellList = {"Buy", "Sell"};

      /*
       * Create all the panels.
       */
      editBasePanel = new BorderedPanel(new BorderLayout(), 
					new Insets(5,5,5,5));
      editPanel = new Panel(new BorderLayout());
      editPanelDecor = new Effect3D(editPanel, "Editing", 
				     Effect3D.RAISED_BORDER);
      editBasePanel.add(editPanelDecor, BorderLayout.CENTER);
      parent.add(editBasePanel, BorderLayout.SOUTH);

      cmdPanel = new BorderedPanel(new FlowLayout(), new Insets(20, 5, 20, 5));
      editPanel.add(cmdPanel, BorderLayout.EAST);
      
      memoPanel = new Panel(new BorderLayout());
      editPanel.add(memoPanel, BorderLayout.CENTER);
      
      fieldPanel = new BorderedPanel(new BorderLayout(), new Insets(0,5,10,5));
      editPanel.add(fieldPanel, BorderLayout.WEST);
      
      uFieldPanel = new Panel(new FlowLayout());
      fieldPanel.add(uFieldPanel, BorderLayout.CENTER);
      lFieldPanel = new Grid(2, 5);
      lFieldPanel.setRuling(0);
      fieldPanel.add(lFieldPanel, BorderLayout.SOUTH);
      
      /*
       * Unchangable fields in uFieldPanel.
       */
      tradeIDLabel = new Label("Trade ID:", Label.RIGHT);
      tradeIDField = new Label("        ", Label.LEFT);
      uFieldPanel.add("tradeIDLabel", tradeIDLabel);	    
      uFieldPanel.add("tradeIDField", tradeIDField);

      acctLabel = new Label("Account:", Label.RIGHT);
      acctField = new Label("               ", Label.LEFT);
      uFieldPanel.add("acctLabel", acctLabel);
      uFieldPanel.add("acctField", acctField);
	    
      /*
       * Changable fields in lFieldPanle.
       */
      buySellLabel = new Label("Buy/Sell:", Label.RIGHT);
      buySellLabel.setAlignment(Label.LEFT);
      buySellField = new Spinner(buySellList, true);
      lFieldPanel.setCell(0, 0, buySellLabel);
      lFieldPanel.setCell(1, 0, buySellField);	    
	    
      tickerLabel = new Label("Ticker:", Label.RIGHT);
      tickerLabel.setAlignment(Label.LEFT);
      tickerField = new TextField();
      //tickerField.setBackground(new java.awt.Color(150, 150, 150));
      lFieldPanel.setCell(0, 1, tickerLabel);
      lFieldPanel.setCell(1, 1, tickerField);	    
	    
      priceLabel = new Label("Price:", Label.RIGHT);
      priceLabel.setAlignment(Label.LEFT);      
      priceField = new TextField();
      //priceField.setBackground(new java.awt.Color(150, 150, 150));
      lFieldPanel.setCell(0, 2, priceLabel);	    
      lFieldPanel.setCell(1, 2, priceField);
	    
      quantityLabel = new Label("Quantity:", Label.RIGHT);
      quantityLabel.setAlignment(Label.LEFT);
      quantityField = new TextField();
      //quantityField.setBackground(new java.awt.Color(150, 150, 150));
      lFieldPanel.setCell(0, 3, quantityLabel);
      lFieldPanel.setCell(1, 3, quantityField);
	    
      trdDateLabel = new Label("Trade Date:", Label.RIGHT);
      trdDateLabel.setAlignment(Label.LEFT);      
      trdDateField = new TextField();
      //trdDateField.setBackground(new java.awt.Color(150, 150, 150));
      lFieldPanel.setCell(0, 4, trdDateLabel);
      lFieldPanel.setCell(1, 4, trdDateField);	    

      //lFieldPanel.setColor(1, Grid.ALL_CELL, Color.white);
      
      /*
       * memoPanel content.
       */
      memoLabel = new Label("Memo:", Label.LEFT);
      memoField = new TextArea("", 2, 20, TextArea.SCROLLBARS_NONE);
      memoPanel.add(memoLabel, BorderLayout.NORTH);
      memoPanel.add(memoField, BorderLayout.CENTER);
	    
      newFlagField = new Checkbox("New Trade");
      memoPanel.add(newFlagField, BorderLayout.SOUTH);
	    
      newFlagField.addItemListener(new NewFlagAdapter());
      
      /*
       * cmdPanel content.
       */
      saveButton = new ImageButton(ITrade.saveImage, "Save");
      saveButton.setPreferredSize(new Dimension(50, 50));
      cmdPanel.add("saveButton", saveButton);

      deleteButton = new ImageButton(ITrade.deleteImage, "Delete");
      deleteButton.setPreferredSize(new Dimension(50, 50));
      cmdPanel.add("deleteButton", deleteButton);

      saveButton.addActionListener(new SaveAdapter());
      deleteButton.addActionListener(new DeleteAdapter());

      try {
          dbRqtSend = msgServer.producerLogin(ITrade.findParameter("dbRequestQueue"), 
				    ITrade.findParameter("dbRequestName"), 
			            ITrade.findParameter("dbRqtSendPWD"), 
				    null, true);
      }
      catch (Exception e) {
	 e.printStackTrace();
      }
      //sendInit();
   }

   void sendInit() {
      try {
	 System.out.println("Sending dbinit");
         dbRqtSend.beginTrans();
         dbRqtSend.sendObj("DBINIT");
         dbRqtSend.sendObj("");
         dbRqtSend.commitTrans();
      }
      catch (Exception e) {
	 e.printStackTrace();
      }
   }   
   
   void setTrans(String acct, String ticker, String id, String bs, 
                 String price, String quantity, String trdDate, String memo) {
      acctField.setText(acct);	 
      tickerField.setText(ticker);
	 
      tradeIDField.setText(id);
      buySellField.setText(bs);
      priceField.setText(price);
      quantityField.setText(quantity);
      trdDateField.setText(trdDate);
      memoField.setText(memo);
      newFlagField.setState(false);	 
   }
   
   void clear() {
      acctField.setText("");	 
      tickerField.setText("");
	 
      tradeIDField.setText("");
      buySellField.setText("");
      priceField.setText("");
      quantityField.setText("");
      trdDateField.setText("");
      memoField.setText("");
      newFlagField.setState(false);	 
   }

   class NewFlagAdapter implements ItemListener {
      public void itemStateChanged(ItemEvent e) {
	 if (newFlagField.getState()) {
	    tradeIDField.setText("");
	 }
	 else {
	    if (tradeIDField.getText().equals("")) {
	       newFlagField.setState(true);
	       ITrade.displayStatus("Please select a transaction to change");
	    }
	 }
      }
   }
   class SaveAdapter implements ActionListener {
      public void actionPerformed(ActionEvent e) {
         if (tickerField.getText().length() <= 0) {
	    ITrade.displayStatus("Please enter a ticker");
	    return;
         }
         if (priceField.getText().length() <= 0) {
	    ITrade.displayStatus("Please enter price");
	    return;
         }
         if (quantityField.getText().length() <= 0) {
            ITrade.displayStatus("Please enter quantity");
            return;
         }
         if (trdDateField.getText().length() <= 0) {
            ITrade.displayStatus("Please enter trade date");
            return;
         }
         if (acctField.getText().length() <= 0) {
            ITrade.displayStatus("Please choose an account");
            return;
         }
        
         /*
          * Send out a insert or update message.
          */
         try {
	    String id;
	    
            dbRqtSend.beginTrans();
	    if (newFlagField.getState()) {
               dbRqtSend.sendObj("INSERT");
	       id = ITrade.findParameter("dbRequestName")+ newIdCount++; 
	    }
	    else {
	       dbRqtSend.sendObj("UPDATE");
	       id = tradeIDField.getText();
	    }
	    dbRqtSend.sendObj(acctField.getText()+"^"+
			      tickerField.getText()+"^"+
			      id+"^"+
                              buySellField.getText()+"^"+
		   	      priceField.getText()+"^"+
                              quantityField.getText()+"^"+
                              trdDateField.getText()+"^"+
                              memoField.getText());
            dbRqtSend.commitTrans();
         }
         catch (Exception ex) {
	    ex.printStackTrace();
         }
         clear();
      }
   }
   
   class DeleteAdapter implements ActionListener {
      public void actionPerformed(ActionEvent e) {
	 if (tradeIDField.getText().equals("")) {
	    return;
	 }
	 
	 //position.deleteCurrentTrans(editing.getTradeID());

         // send delete message here.

         /*
          * Send out a insert or update message.
          */
         try {
            dbRqtSend.beginTrans();
            dbRqtSend.sendObj("DELETE");
	    dbRqtSend.sendObj(acctField.getText()+"^"+
			      tickerField.getText()+"^"+
			      tradeIDField.getText()+"^"+
                              buySellField.getText()+"^"+
		   	      priceField.getText()+"^"+
                              quantityField.getText()+"^"+
                              trdDateField.getText()+"^"+
                              memoField.getText());
	    dbRqtSend.commitTrans();
         }
         catch (Exception ex) {
	    ex.printStackTrace();
         }

	 clear();
      }
   }
}
