/*
 * Copyright (c) 1996-1997, InetSoft Technology Corp, All Rights Reserved.
 *
 * The software and information contained herein are copyrighted and 
 * proprietary to InetSoft Technology Corp. This software is furnished 
 * pursuant to a written license agreement and may be used, copied, 
 * transmitted, and stored only in accordance with the terms of such 
 * license and with the inclusion of the above copyright notice. Please 
 * refer to the file "COPYRIGHT" for further copyright and licensing 
 * information. This software and information or any other copies 
 * thereof may not be provided or otherwise made available to any other 
 * person. 
 */
package itrade.gui;

import java.awt.*;
import java.awt.event.*;
import java.rmi.*;
import java.rmi.server.*;
import tea.set.*;
import bamboo.pipe.*;

class Message {
   public Effect3D       msgPanelDecor;
   public Panel          msgPanel;
   public Panel          msgBasePanel;   
   public BorderedPanel  sendPanel;
   public BorderedPanel  cmdPanel;
   
   public Label     sendMsgLabel;
   public TextField sendMsgInput;
   public TextArea  msgList;
   public ImageButton  clearButton;
   public ImageButton  msgStopButton;

   public ConsumerConfig    msgReceive;
   public Producer          msgSend;
   public ConsumerNotify    msgHandler;

   public String            cache;
   
   Message(String layoutPos, Panel parent, BambooPipe msgServer) { 
      msgBasePanel = new BorderedPanel(new BorderLayout(), 
				 new Insets(10,0,10,0));
      
      msgPanel = new BorderedPanel(new BorderLayout(),
					new Insets(0, 5, 0, 0));
      msgPanel.setBackground(new java.awt.Color(192, 192, 192));
      msgPanelDecor = new Effect3D(msgPanel, " Messages ", 
                                   Effect3D.RAISED_BORDER);
      msgBasePanel.add(msgPanelDecor, BorderLayout.CENTER);
      parent.add(msgBasePanel, layoutPos);

      msgList = new TextArea(1, 50);
      msgList.setEditable(false);
      msgPanel.add(msgList, BorderLayout.CENTER);

      sendPanel = new BorderedPanel(new BorderLayout(), 
				    new Insets(5, 10, 5, 10));
      sendMsgLabel = new Label("Send Message:");
      sendPanel.add(sendMsgLabel, BorderLayout.WEST);

      sendMsgInput = new TextField(0);
      sendPanel.add(sendMsgInput, BorderLayout.CENTER);
      msgPanel.add(sendPanel, BorderLayout.SOUTH);
      
      cmdPanel = new BorderedPanel(new VFlowLayout(VFlowLayout.CENTER | 
						   VFlowLayout.V_CENTER), 
				   new Insets(3, 3, 3, 3));
      clearButton = new ImageButton(ITrade.clearImage, "Clear");
      clearButton.setPreferredSize(new Dimension(40, 40)); 
      cmdPanel.add("clearButton", clearButton);

      msgStopButton = new ImageButton(ITrade.stopImage, "Stop");
      msgStopButton.setPreferredSize(new Dimension(40, 40));
      msgStopButton.setImage2(ITrade.startImage);
      msgStopButton.setToggle(true);
      cmdPanel.add("clearButton", msgStopButton);
      msgPanel.add(cmdPanel, BorderLayout.EAST);
      
      /*
       * install GUI event handler.
       */
      clearButton.addActionListener(new MsgClearAdapter());
      msgStopButton.addActionListener(new MsgStopAdapter());
      sendMsgInput.addActionListener(new MsgSendAdapter());	 

      /*
       * Logon to message queues. There are a total of four queues.
       * #2 message queue: producer-clients, consumer-clients	  
       */
       try {
          msgHandler = new MsgHandler();

          msgSend = msgServer.producerLogin(ITrade.findParameter("msgQueue"), 
				      ITrade.findParameter("msgName"), 
			              ITrade.findParameter("msgSendPWD"), 
				      null, true);
          msgReceive = msgServer.consumerLogin(ITrade.findParameter("msgQueue"), 
				   ITrade.findParameter("msgName"), 
			           ITrade.findParameter("msgReceivePWD"), 
				   (ConsumerNotify)msgHandler, true);
          msgReceive.readyReceive();
       }
       catch (Exception e) {
	  e.printStackTrace();
       }
       
   }

   class MsgClearAdapter implements ActionListener {
      public void actionPerformed(ActionEvent e) {
	 msgList.setText("");
      }
   }
   
   class MsgStopAdapter implements ActionListener {
      public void actionPerformed(ActionEvent e) {
	 boolean toggled;
	 if ((toggled = msgStopButton.getState()) == true) {
	    cache = msgList.getText();
	    msgStopButton.setLabel("Start");
	 }
	 else {
	    msgList.setText(cache);
	    cache = null;
	    msgStopButton.setLabel("Stop");	    
	 }
      }
   }

   class MsgSendAdapter implements ActionListener {
      public void actionPerformed(ActionEvent e) {
	 try {
            msgSend.beginTrans();
	    msgSend.sendObj(sendMsgInput.getText());
	    msgSend.commitTrans();
	    sendMsgInput.setText("");
	 }
	 catch (Exception ex) {
	    ex.printStackTrace();
	 }
      }
   }

   class MsgHandler extends UnicastRemoteObject
     implements ConsumerNotify, java.io.Serializable {
      String newMsg;

      public MsgHandler() throws RemoteException {
	 super();
      }
	
      public void consumerNotify(Consumer c) {
	 String producer;
	 
	 try {
	    c.beginTrans();
	    newMsg = (String)c.receiveObj();
	    producer = c.getProducerName();
	    if (cache == null) {
	       msgList.setText(msgList.getText()+producer+":"+newMsg+"\n");
	    }
	    else {
	       cache = cache + producer+":"+newMsg + "\n";
	    }
	    c.commitTrans();
	 }
	 catch (Exception e) {
	    System.out.println("BambooClient exception:"+e.getMessage());
	    e.printStackTrace();
         }
      }
   }
}
