/*
 * Copyright (c) 1996-1997, InetSoft Technology Corp, All Rights Reserved.
 *
 * The software and information contained herein are copyrighted and 
 * proprietary to InetSoft Technology Corp. This software is furnished 
 * pursuant to a written license agreement and may be used, copied, 
 * transmitted, and stored only in accordance with the terms of such 
 * license and with the inclusion of the above copyright notice. Please 
 * refer to the file "COPYRIGHT" for further copyright and licensing 
 * information. This software and information or any other copies 
 * thereof may not be provided or otherwise made available to any other 
 * person. 
 */
package itrade.gui;

import java.awt.*;
import java.awt.event.*;
import java.rmi.*;
import java.rmi.server.*;
import tea.set.*;
import bamboo.pipe.*;

class NewsTicker {   
   public BorderedPanel       tickerPanel;
   public Panel               newsPanel;
   public BorderCenterLayout  tickerLayout;
   
   public TickerTape  tickerTape;
   public Image       stopImage;
   public Image       startImage;   
   public ImageButton tickerStopButton;
   public Image[]     majorNewsImages;
   public Animator    majorNewsLight;
   public Label       majorNewsLabel;

   public ConsumerConfig   newsReceive;
   public ConsumerNotify   newsHandler;
   
   NewsTicker(String layoutPos, Panel parent, BambooPipe msgServer) {
      tickerLayout = new BorderCenterLayout();
      tickerLayout.setHgap(5);
      tickerPanel = new BorderedPanel(tickerLayout, 
				      new Insets(5, 10, 5, 10));

      stopImage = ITrade.stopImage;
      startImage = ITrade.startImage;
      tickerStopButton = new ImageButton(stopImage);
      tickerStopButton.setImage2(startImage);
      tickerStopButton.setToggle(true);
      tickerPanel.add(tickerStopButton, 
         new BorderCenterLayout.Constraints("West", 0));

	   
      tickerTape = new TickerTape("Price Tape will be displayed here");
      tickerTape.setBackground(Color.black);
      tickerTape.setForeground(Color.green);
      tickerTape.setName("tickerTape");
      tickerPanel.add(tickerTape,
     new BorderCenterLayout.Constraints("Center",BorderCenterLayout.V_STATIC));
      
      newsPanel = new Panel();
      
      majorNewsImages = new Image[2];
      majorNewsImages[0] = ITrade.blackBallImage;
      majorNewsImages[1] = ITrade.redBallImage;
      majorNewsLight = new Animator(majorNewsImages);
      newsPanel.add("majorNewsLight", majorNewsLight);
      majorNewsLabel = new java.awt.Label("Major News");
      newsPanel.add("majorNewsLabel", majorNewsLabel);
      tickerPanel.add(newsPanel,
          new BorderCenterLayout.Constraints("East", 0));
      
      majorNewsLight.setDelay(1000);
      majorNewsLight.start();
      newsPanel.setVisible(false);
      
      parent.add(layoutPos, tickerPanel);
      
      tickerStopButton.addActionListener(new TickerStopAdapter());

      /*
       * Logon to the news queue.
       * news queue: producer-news server, consumer-clients.
       */
      try {
         newsHandler = new NewsHandler();

         newsReceive = msgServer.consumerLogin(ITrade.findParameter("newsQueue"), 
       			ITrade.findParameter("newsName"), 
      		        ITrade.findParameter("newsPWD"), 
       			(ConsumerNotify)newsHandler, true);
         newsReceive.readyReceive();
      }
      catch (Exception e) {
	 System.out.println("news consumer failed");
	 e.printStackTrace();
      }
      tickerTape.start();
   }

   class TickerStopAdapter implements ActionListener {
      public void actionPerformed(ActionEvent e) {
	 boolean toggled;
	 if ((toggled = tickerStopButton.getState()) == true) {
	    tickerTape.stop();
	 }
	 else {
	    tickerTape.start();
	 }
      }
   }
   
   class NewsHandler extends UnicastRemoteObject
     implements ConsumerNotify, java.io.Serializable {
      String newMsg;
      
      public NewsHandler() throws java.rmi.RemoteException {
	 super();
      }

      public void consumerNotify(Consumer c) throws java.rmi.RemoteException {
	 try {
	    c.beginTrans();
	    newMsg = (String)c.receiveObj();
	    c.commitTrans();
	    if (newMsg.charAt(0) == '^') {
	       majorNewsLight.start();
	       newsPanel.setVisible(true);
	       newMsg = newMsg.substring(1);
	       tickerPanel.validate();       
	    }
	    tickerTape.setMessage(newMsg);
	    if (newsPanel.isVisible()) {
	       try {
	          Thread.sleep(5 * 1000);
	       }
	       catch (InterruptedException e) {
	       }
	       majorNewsLight.stop();
	       newsPanel.setVisible(false);
	       tickerPanel.validate();
	    }
	 }
	 catch (BPTransException e) {
	    System.out.println("BambooClient exception:"+e.getMessage());
	 }
      }
   }
}
