/*
 * Copyright (c) 1996-1997, InetSoft Technology Corp, All Rights Reserved.
 *
 * The software and information contained herein are copyrighted and 
 * proprietary to InetSoft Technology Corp. This software is furnished 
 * pursuant to a written license agreement and may be used, copied, 
 * transmitted, and stored only in accordance with the terms of such 
 * license and with the inclusion of the above copyright notice. Please 
 * refer to the file "COPYRIGHT" for further copyright and licensing 
 * information. This software and information or any other copies 
 * thereof may not be provided or otherwise made available to any other 
 * person. 
 */
package itrade.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.rmi.*;
import java.rmi.server.*;
import tea.set.*;
import bamboo.pipe.*;

class Position {
   public Panel             positionPanel;
   public BorderShareLayout positionLayout;
   
   public Panel       cmdPanel;   
   public Grid        flagGrid;
   public Effect3D    flagDecor;   
   
   public Panel       switchPanel;
   public Transaction transaction;
   public Composition composition;

   public PositionTree positionTree;
   
   public ImageButton    refreshButton;
   public ImageButton    transButton;

   public ConsumerConfig       dbRqtReceive;
   /* public ConsumerConfig       dbReplyReceive; */
   public ConsumerNotify       dbRqtHandler;
   /* public ConsumerNotify       dbReplyHandler; */
   
   public Image       okStatus;
   public Image       errStatus;
   public Image       pendingSaveStatus;
   public Image       pendingDeleteStatus;
   public ImageLabel  okFlag;  
   public ImageLabel  errFlag;
   public ImageLabel  pendingSaveFlag;
   public ImageLabel  pendingDeleteFlag;   
   
   /* holds the current position on the tree. */ 
   public Node          currentPos;     

   Position(Panel parent, ItemListener transSelectAdapter, 
	    ActionListener refreshAdapter, BambooPipe msgServer) {
      /*
       * The main panel, positionPanel
       */
      positionLayout = new BorderShareLayout();
      positionPanel = new BorderedPanel(positionLayout,
					new Insets(0, 5, 0 ,0));
      positionLayout.setHgap(3);
      parent.add(positionPanel, BorderLayout.CENTER);

      /*
       * The switch panel and two objects on it: transaction and composition
       */
      switchPanel = new Panel(new OverlapLayout());
      positionPanel.add(switchPanel, 
	      new BorderShareLayout.Constraints(BorderLayout.CENTER, true)); 
      transaction = new Transaction(switchPanel, transSelectAdapter);
      composition = new Composition(switchPanel);

      /*
       * positionTree is on the main panel.
       */
      positionTree = new PositionTree(positionPanel, new PosSelectAdapter());
      
      /*
       * The command panel stuff.
       */
      cmdPanel = new BorderedPanel(new VFlowLayout(VFlowLayout.CENTER |
						   VFlowLayout.V_CENTER),
				   new Insets(3, 3, 3, 3));
      positionPanel.add(cmdPanel, 
	     new BorderShareLayout.Constraints(BorderLayout.EAST, false));

      flagGrid = new Grid(4, 1);
      flagGrid.setRuling(Grid.ALL_CELL, Grid.ALL_CELL, Grid.NONE);
      okStatus = ITrade.greenDialImage;
      okFlag = new ImageLabel(okStatus, "Ok");
      errStatus = ITrade.redDialImage;
      errFlag = new ImageLabel(errStatus, "Error");
      pendingSaveStatus = ITrade.yellowDialImage;
      pendingSaveFlag = new ImageLabel(pendingSaveStatus, "Pending Save"); 
      pendingDeleteStatus = ITrade.blackDialImage;
      pendingDeleteFlag = new ImageLabel(pendingDeleteStatus, "Pending Del");
      flagGrid.setCell(0, 0, okFlag);
      flagGrid.setCell(1, 0, errFlag);
      flagGrid.setCell(2, 0, pendingSaveFlag);
      flagGrid.setCell(3, 0, pendingDeleteFlag);
      flagGrid.setGap(Grid.ALL_CELL, 0, new Insets(2, 3, 2, 3));
      flagDecor = new Effect3D(flagGrid, "Trans Status", 
			       Effect3D.LOWERED_BORDER);
      cmdPanel.add("flagGrid", flagDecor);
      
      refreshButton = new ImageButton(ITrade.refreshImage, "Refresh");
      refreshButton.setPreferredSize(new Dimension(50, 50));
      cmdPanel.add("refreshButton", refreshButton);

      transButton = new ImageButton(ITrade.graphicImage, "Graphic");
      transButton.setPreferredSize(new Dimension(50, 50));
      transButton.setToggle(true);
      transButton.setImage2(ITrade.detailImage);
      cmdPanel.add("transButton", transButton);

      System.out.println("Binding refresh");
      refreshButton.addActionListener(refreshAdapter);
      transButton.addActionListener(new TransDisplayAdapter());

      
      /*
       * Log on to the two transaction/database related queues.
       * db send queue: producer-client, consumer-clients/db server
       * db receive queue: producer-db server, consumer-clients
       */
       try {
          dbRqtHandler = new DBRqtHandler();
          /* dbReplyHandler = new DBReplyHandler();	   */

          dbRqtReceive =msgServer.consumerLogin(ITrade.findParameter("dbRequestQueue"),
				ITrade.findParameter("dbRequestName"), 
			        ITrade.findParameter("dbRqtReceivePWD"), 
				(ConsumerNotify)dbRqtHandler, true);

	  /*
          dbReplyReceive = 
	     msgServer.consumerLogin(ITrade.findParameter("dbReplyQueue"), 
				ITrade.findParameter("dbReplyName"), 
			        ITrade.findParameter("dbReplyReceivePWD"), 
				(ConsumerNotify)dbReplyHandler, true);
				*/
          dbRqtReceive.readyReceive();
	  /*
          dbReplyReceive.readyReceive();  
	  */
       }
       catch (Exception e) {
	  e.printStackTrace();
       }
   }

   String getObject(int r , int c) {
      return transaction.getObject(r, c);
   }
   
   public void setupTransaction(Node posNode) {
      String[]  transList;
      int       i, rCount, diff;

      if (posNode == null) {
         transaction.clear();
         return;
      }

      /*
       * Setup the transaction grid display.
       */
      currentPos = posNode;
      transaction.setupData((String)posNode.getUserData());
   }

   public void setupComposition(Node acctNode) { 
      Vector xData = new Vector();
      Vector yData = new Vector();
      int    cCount = acctNode.getChildCount();
      int    i;
      
      for (i = 0; i < cCount; i++) {
	 String[] pos = 
	    Tool.tokenize((String)acctNode.getChild(i).getUserData(), "^");
	 String[] posDesc = Tool.tokenize(pos[0], "|");
	 System.out.println("add Y:"+posDesc[0]+" X:"+posDesc[1]
	                    +"*"+posDesc[3]);
	 yData.addElement(new Double(Double.valueOf(posDesc[1]).doubleValue()
				* Double.valueOf(posDesc[3]).doubleValue()));
	 xData.addElement(posDesc[0]);
      }
      composition.setupData("Account:"+acctNode.getPath(), acctNode,
			    xData, yData);
   }
   

   String getCurrentPos() {
      //String[] pos = 
      //    Tool.tokenize((String)currentPos.getPath(), "^");
      //return pos[0];
      if (currentPos != null) {
         return currentPos.getPath();
      }
      return null;
   }
   
   void deleteCurrentTrans(String transID) {
      if (transaction.deleteCurrent(transID) ==0) {
	 positionTree.deleteCurrent(currentPos);
      }
   }
   
   class PosSelectAdapter implements ItemListener {
      public void itemStateChanged(ItemEvent e) {
	 Object   tmp;
	 Node     selectedNode;
	 
	 if (e.getStateChange() != ItemEvent.SELECTED) {
	    return;
	 }
	    
	 if (!((tmp = e.getItem()) instanceof Node)) {
	    return;
	 }
	 else {
	    selectedNode = (Node)tmp;
         }
	 
         /*
          * A selection on a acct node or a position node will 
	  * change the display of transaction and composition. Otherwise,
	  * the positions will be wiped out.
	  */
	 if (selectedNode.isLeaf()) {
            String       parentString = selectedNode.getPath();
            int          idx = parentString.lastIndexOf('.');
	    
	    /*
	     * waiting for refresh has no '.'
	     */
	    if (idx < 0) {
	       return;
	    }
            parentString = parentString.substring(0, idx);
            Node  parentNode = positionTree.findNode(parentString);

            setupTransaction(selectedNode);
            setupComposition(parentNode);
	 }
         else {
            if (selectedNode.getChild(0).isLeaf()) {
	       setupTransaction(null);
               setupComposition(selectedNode);
	    }
            else {
	       setupTransaction(null);
               setupComposition(null);
	    }
	 }
      }
   } /* PosSelectAdapter */


   class TransDisplayAdapter implements ActionListener {
      public void actionPerformed(ActionEvent e) {
	 boolean toggled;
	 String[]  transList;
 	 
	 if ((toggled = transButton.getState()) == true) {
	    transButton.setLabel("Trans");

	    composition.setVisible(true);
	    transaction.setVisible(false);
	 }
	 else {
	    transButton.setLabel("Graphic");

	    composition.setVisible(false);
	    transaction.setVisible(true);
	 }
      }
   }

   class DBRqtHandler extends UnicastRemoteObject
            implements ConsumerNotify, java.io.Serializable {
      public DBRqtHandler() throws RemoteException {
	 super();
      }
	
      public void consumerNotify(Consumer c) {
	 String    code;
	 Object    data;
	 Node      posNode = null;
	 
	 try {
	    c.beginTrans();
	    code = (String)c.receiveObj();
	    data = c.receiveObj();
	    c.commitTrans();
	    System.out.println("code:"+code+"\ndata:"+data);
	 
   	    if (code.equalsIgnoreCase("initReply")) {
   	       System.out.println("processInit of:"+data);
	       //try { Thread.sleep(10000); }
	       //catch (Exception e) {}
	       posNode = positionTree.processInit((String[])data);
	    }
	    else if (code.equalsIgnoreCase("insertReply")
	          || code.equalsIgnoreCase("updateReply")
	          || code.equalsIgnoreCase("deleteReply")) {
	       posNode = positionTree.processReply(code+"^"+(String)data);
	       //transaction.setupData(newTrans);
	    }
	    else if (code.equalsIgnoreCase("INSERT")) {
               posNode = positionTree.insertTrans("pendingSave^"+(String)data);
	    }
	    if (code.equalsIgnoreCase("UPDATE")) {
               posNode = positionTree.updateTrans("pendingSave^"+(String)data);
	    }
	    if (code.equalsIgnoreCase("DELETE")) {
               posNode = positionTree.deleteTrans("pendingDelete^"+
						  (String)data);
	    }

	    if (posNode == null)
	       return;

	    String pos = getCurrentPos();
	    if (pos != null && pos.equals(posNode.getPath())) {
	       setupTransaction(posNode);
	    }

	    String acctPath = posNode.getPath();
	    int    cut = acctPath.lastIndexOf('.');
	    acctPath = acctPath.substring(0, cut); 
	    Node  acctNode = positionTree.findNode(acctPath);
	    if (composition.getCurrentAcct().equals(acctPath)) {
	       setupComposition(acctNode);
	    }
	 }
	 catch (Exception e) {
	    System.out.println("DB message process exception:"+e.getMessage());
	    e.printStackTrace();
	    return;
         }
      } /* consumerNotify */
   }
   
   /*
   class DBReplyHandler extends UnicastRemoteObject
           implements ConsumerNotify, java.io.Serializable {
      String code;
      Object data;
	
      public DBReplyHandler() throws RemoteException {
	 super();
      }
	
      public void consumerNotify(Consumer c) {
	 try {
	    c.beginTrans();
	    code = (String)c.receiveObj();
	    data = c.receiveObj();
	    c.commitTrans();
	 }
	 catch (Exception e) {
	    System.out.println("BambooClient exception:"+e.getMessage());
         }
	 
	 if (code.equalsIgnoreCase("initReply")) {
	    System.out.println("processInit of:"+data);
	    //try { Thread.sleep(10000); }
	    //catch (Exception e) {}
	    positionTree.processInit((String[])data);
	 }
	 else if (code.equalsIgnoreCase("insertReply")
	       || code.equalsIgnoreCase("updateReply")
	       || code.equalsIgnoreCase("deleteReply")) {
	    String newTrans = positionTree.processReply(code+"^"+(String)data);
	    transaction.setupData(newTrans);
	 }
      }
   }
   */
}




