/*
 * Copyright (c) 1996-1997, InetSoft Technology Corp, All Rights Reserved.
 *
 * The software and information contained herein are copyrighted and 
 * proprietary to InetSoft Technology Corp. This software is furnished 
 * pursuant to a written license agreement and may be used, copied, 
 * transmitted, and stored only in accordance with the terms of such 
 * license and with the inclusion of the above copyright notice. Please 
 * refer to the file "COPYRIGHT" for further copyright and licensing 
 * information. This software and information or any other copies 
 * thereof may not be provided or otherwise made available to any other 
 * person. 
 */
package itrade.gui;

import java.awt.*;
import java.awt.event.*;
import java.rmi.*;
import java.rmi.server.*;
import tea.set.*;
import bamboo.pipe.*;

class PositionTree {
   private Effect3D    treeDecor;
   private ForestGrid  tree;
   private Panel       swithPanel;

   PositionTree(Panel parent, ItemListener posSelectAdapter) {
      String[] posTitle = {"Ticker", "Quantity", "Name", "Markup"};

      tree = new ForestGrid();
      tree.setStyle(Forest.LINE);
      //tree.setRowSelectable(true);
      treeDecor = new Effect3D(new Scroller(tree, true, 200, 100), "Portfolio",
                               Effect3D.LOWERED_BORDER);
      tree.setColCount(4);
      tree.setGap(Grid.ALL_CELL, Grid.ALL_CELL, new Insets(1,0,1,0));
      tree.setColHeader(posTitle);
      tree.setColWidth(Grid.ALL_CELL, 50);
      tree.setForestRuling(Grid.VERTICAL);
      tree.add("Waiting for initialization");
      parent.add(treeDecor, 
	  new BorderShareLayout.Constraints(BorderLayout.WEST, true));

      tree.addItemListener(posSelectAdapter);
      
      //processInit("");
   }
   
   Node processInit(String[] initPos) {
      String[] posList;
      String[] transList;
      String[] descFields;
      String[] transFields;
      Node     posNode;
      String   selPath;
      int      row;
      int i, j;

      tree.remove("Waiting for initialization");	 
      
      /*
       * initPos is a string array of accts. Every acct has a format of: 
       *   acct_path~pos1~pos2~pos3. 
       * Then each position has a format of:
       *   pos_desc^trans1^trans2^trans3
       * pos_desc and trans all have a format of:
       *   field1|field2|field3
       * pos desc: ticker|share|description|markup price
       * trans: trade id|buy/Sell|price|quantity|trdDate|memo
       *
       * The leaf node of the tree is position with the ticker as the label.
       * Each row in the table is the description of the position.
       */
        
      for (i = 0; i < initPos.length; i++) {
         posList = Tool.tokenize(initPos[i], "~");
           
         tree.findNode(posList[0], true);
         for (j = 1; j < posList.length; j++) {
            transList = Tool.tokenize(posList[j], "^");
              
            descFields = Tool.tokenize(transList[0], "|");
            System.out.println("Add Node:"+posList[0]+"."+descFields[0]);
            posNode = tree.findNode(posList[0]+"."+descFields[0], true);
            row = tree.getRowIndex(posNode.getPath());
            for (int k = 1; k < descFields.length; k++) {
               System.out.println("Add field:"+descFields[k]+" to row:"+row);
            }
            tree.setRow(row, descFields);
            posNode.setUserData(posList[j]);
         }
      }
      //tree.invalidate();
      tree.validate();
      tree.repaint();

      if ((selPath = tree.getSelectedPath()) != null) {
         posNode = findNode(selPath);
      }
      else {
	 posNode = null;
      }
      
      return posNode;
   }
            
   Node findNode(String parentString) {
      return tree.findNode(parentString, false);
   }
   
   void deleteCurrent(Node currentPos) {
      tree.remove(currentPos.getPath());
   }

   /*
    * data has a format:
    *  acct^ticker^tradeID^B/S^Price^Quantity^TradeDate^Memo
    */
   Node insertTrans(String data) {
      String[] fields = Tool.tokenize(data, "^");
      Node     acctNode;
      Node     posNode;
      String   transList;
      String[] desc = new String[4];
      boolean  newPos = false;
      
      acctNode = findAcctNode(fields[1]);
      System.out.println("acctNode"+acctNode+"key:"+fields[1]);
      posNode = tree.findNode(acctNode.getPath()+"."+fields[2], true);
      if ((transList =(String)posNode.getUserData()) == null) {
	 newPos = true;
	 transList = fields[2]+"|"+fields[6]+"|"+fields[2]+"|"+fields[5];
      }
      transList = transList+"^"+fields[0]+"|"+fields[3]+"|"+fields[4]+"|"
          +fields[5]+"|"+fields[6]+"|"+fields[7];
      if (fields.length >= 9) {
	 transList = transList + "|" + fields[8];
      }

      posNode.setUserData(transList);

      if (newPos) {
         desc[0] = fields[2];
         desc[1] = fields[6];
         desc[2] = fields[2];
         desc[3] = fields[5];
         tree.setRow(tree.getRowIndex(posNode.getPath()), desc);
      }
      
      return(posNode);
   }

   Node updateTrans(String data) {
      String[] fields = Tool.tokenize(data, "^");
      Node     acctNode;
      Node     posNode;
      String   newTrans;
      String   newList;
      String[] transList;
      int      i;
      
      acctNode = findAcctNode(fields[1]);
      posNode = tree.findNode(acctNode.getPath()+"."+fields[2], false);
      transList = Tool.tokenize((String)posNode.getUserData(), "^");
      newTrans = fields[0]+"|"+fields[3]+"|"+fields[4]+"|"+fields[5]+
	          "|"+fields[6]+"|"+fields[7];
      if (fields.length >= 9) {
	 newTrans = newTrans + "|" + fields[8];
      }
      for (i = 1, newList = transList[0]; i < transList.length; i++) {
	 String[]  oldFields = Tool.tokenize(transList[i], "|");
	 if (oldFields[1].equals(fields[3])) {
	    newList = newList + "^" +newTrans;
	 }
	 else {
	    newList = newList + "^" + transList[i];
	 }
      }
      System.out.println("newList:"+newList);	    
      posNode.setUserData(newList);
      
      return(posNode);
   }
   
   Node deleteTrans(String data) {
      String[] fields = Tool.tokenize(data, "^");
      Node     acctNode;
      Node     posNode;
      String   newTrans;
      String   newList;
      String[] transList;
      int      i;
      
      acctNode = findAcctNode(fields[1]);
      System.out.println("acctNode:"+acctNode);
      posNode = tree.findNode(acctNode.getPath()+"."+fields[2], false);
      transList = Tool.tokenize((String)posNode.getUserData(), "^");
            System.out.println("oldList:"+posNode.getUserData());
      for (i = 1, newList = transList[0]; i < transList.length; i++) {
	 System.out.println("trans:"+transList[i]);
	 String[]  oldFields = Tool.tokenize(transList[i], "|");
	 if (oldFields[1].equals(fields[3])) {
            newTrans = fields[0]+"|"+oldFields[1]+"|"+oldFields[2]+"|"
                  +oldFields[3]+"|"+oldFields[4]+"|"+oldFields[5];
	    if (oldFields.length >= 7) {
	       newTrans = newTrans + "|" + oldFields[6];
	    }
	    newList = newList + "^" + newTrans;
	 }
	 else {
	    newList = newList + "^" + transList[i];
	 }
      }
      System.out.println("newList:"+newList);
	    
      posNode.setUserData(newList);
      
      return(posNode);
   }

   private Node findAcctNode(String acct) {
      int  rootCount;
      Node acctNode;
      
      rootCount = tree.getRootCount();
      for (int i = 0; i < rootCount; i++) {
	 acctNode = findAcctInTree(tree.getRoot(i), acct);
	 if (acctNode != null) {
	    return(acctNode);
	 }
      }
      return null;
   }
   
   private Node findAcctInTree(Node root, String acct) {
      Node child;
      int  childCount = root.getChildCount();
      Node acctNode;
      
      for (int i = 0; i < childCount; i++) {
	 child = root.getChild(i);

	 if (child.getPath().indexOf(acct) >= 0) {
	    return child;
	 }
	 if (!child.isLeaf()) {
	    if ((acctNode = findAcctInTree(child, acct)) != null) {
	       return acctNode;
	    }
	 }
      }
      return null;
   }

   /*
    * Reply message has a format of:
    * status^acct^ticker^tradeID^msg^tempTradeID
    */
   Node processReply(String reply) {
      String[] fields = Tool.tokenize(reply, "^");
      Node     acctNode;
      Node     posNode;
      String   newTrans;
      String   newList;
      String[] transList;
      String   newMsg;
      int      i;
      
      System.out.println("reply:"+reply);
      acctNode = findAcctNode(fields[2]);
      posNode = tree.findNode(acctNode.getPath()+"."+fields[3], false);
      transList = Tool.tokenize((String)posNode.getUserData(), "^");
      System.out.println("transList:"+transList);
      for (i = 1, newList = transList[0]; i < transList.length; i++) {
	 String[]  oldFields = Tool.tokenize(transList[i], "|");
	 System.out.println("flen:"+fields.length+" olen:"+oldFields.length);
	 if (oldFields.length >= 7) {
	    newMsg = oldFields[6] + (fields.length >= 6 ? fields[5] : "");
	 }
	 else {
	    newMsg = fields.length >= 6 ? fields[5] : "";
	 }
	    
	 System.out.println("code:"+fields[0]+" ID:"+fields[4]+"-"+fields[6]);
	 if (fields[0].equalsIgnoreCase("insertReply")) {
	    if (oldFields[1].equals(fields[6])) {
               newTrans = fields[1]+"|"+fields[4]+"|"+oldFields[2]+"|"
                     +fields[3]+"|"+oldFields[4]+"|"+oldFields[5]+"|"
                     +newMsg;
	       newList = newList + "^" + newTrans;
	    }
	    else {
	       newList = newList + "^" + transList[i];
	    }
	 }
	 else if (fields[0].equalsIgnoreCase("updateReply")) {
	    if (oldFields[1].equals(fields[4])) {
               newTrans = fields[1]+"|"+oldFields[1]+"|"+oldFields[2]+"|"
                     +oldFields[3]+"|"+oldFields[4]+"|"+oldFields[5]+"|"
                     +newMsg;
	       newList = newList + "^" + newTrans;
	    }
	    else {
	       newList = newList + "^" + transList[i];
	    }
	 }
	 else if (fields[0].equalsIgnoreCase("deleteReply")) {
	    if (oldFields[1].equals(fields[4])) {
	       if (fields[1].equals("ok")) {
		  continue;
	       }
	       else {
                  newTrans = fields[1]+"|"+oldFields[1]+"|"+oldFields[2]+"|"
                        +oldFields[3]+"|"+oldFields[4]+"|"+oldFields[5]+"|"
                        +newMsg;
	          newList = newList + "^" + newTrans;
	       }
	    }
	    else {
	       newList = newList + "^" + transList[i];
	    }
	 }
      }
	    
      System.out.println("newList:"+newList);
      posNode.setUserData(newList);
      
      return posNode;
   }
}
