/*
 * Copyright (c) 1996-1997, InetSoft Technology Corp, All Rights Reserved.
 *
 * The software and information contained herein are copyrighted and 
 * proprietary to InetSoft Technology Corp. This software is furnished 
 * pursuant to a written license agreement and may be used, copied, 
 * transmitted, and stored only in accordance with the terms of such 
 * license and with the inclusion of the above copyright notice. Please 
 * refer to the file "COPYRIGHT" for further copyright and licensing 
 * information. This software and information or any other copies 
 * thereof may not be provided or otherwise made available to any other 
 * person. 
 */
package itrade.gui;

import java.awt.*;
import java.awt.event.*;
import java.rmi.*;
import java.rmi.server.*;
import tea.set.*;
import bamboo.pipe.*;
   
class Transaction {
   /*
    * GUI objects on the transaction detail panel
    */
   private Effect3D    transPanelDecor;
   private Panel       transPanel;   
   private Scroller    transGridScroll;
   private TextGrid    transGrid;   

   private Image       okStatus;
   private Image       errStatus;
   private Image       pendingSaveStatus;
   private Image       pendingDeleteStatus;
   
   Transaction(Panel parent, ItemListener transSelect) {
      String title[] = {"", "Trade ID", "Buy/Sell","Price",
		        "Quantity", "Trade Date", "Memo"};

      okStatus = ITrade.greenDialImage;
      errStatus = ITrade.redDialImage;
      pendingSaveStatus = ITrade.yellowDialImage;
      pendingDeleteStatus = ITrade.blackDialImage;

      transPanel = new Panel(new BorderLayout());
      transPanelDecor = new Effect3D(transPanel, "Transactions", 
			             Effect3D.LOWERED_BORDER);
      transGrid = new TextGrid(100, 7);
      transGrid.setGap(Grid.ALL_CELL, Grid.ALL_CELL, 
	   	       new Insets(1,0,1,0));
      transGrid.setColHeader(title);
      transGrid.setRowSelectable(true);
      transGrid.setRowCharHeight(Grid.ALL_CELL, 1);
      transGrid.setColCharWidth(Grid.ALL_CELL, 10);
      transGrid.setColCharWidth(0, 2);      
      transGridScroll = new Scroller(transGrid, true, 223,190);
      
      transPanel.add(transGridScroll, BorderLayout.CENTER);
      //transPanelDecor.setBounds(253, 9, 225, 216);

      parent.add("transPanelDecor", transPanelDecor);
      transPanel.setVisible(true);            

      transGrid.addItemListener(transSelect);
   }
   
   void clear() {
      transGrid.setRowCount(0);   
   }
   
   void setupData(String input) {
      String[]  transList;
      int       i, rCount, diff;
      String[]  transFields;
      
      transList = Tool.tokenize(input, "^");
      rCount = transGrid.getRowCount();
	 
      if ((diff = rCount - (transList.length - 1)) > 0) {
	 transGrid.removeRow((transList.length -1), diff);
      }
      else if (diff < 0) {
         transGrid.addRow(-diff);
      }

      for (i = 1; i < transList.length; i++) {
         transGrid.setRow(i - 1, transFields = 
			            Tool.tokenize(transList[i], "|"));
	 if (transFields[0].equals("ok")) {
	    transGrid.setObject(i - 1, 0, okStatus);
	 }
	 else if (transFields[0].equals("error")) {
	    transGrid.setObject(i - 1, 0, errStatus);
	 }
	 if (transFields[0].equals("pendingSave")) {
	    transGrid.setObject(i - 1, 0, pendingSaveStatus);
	 }
	 if (transFields[0].equals("pendingDelete")) {
	    transGrid.setObject(i - 1, 0, pendingDeleteStatus);
	 }
      }
   }
   
   String getObject(int row, int col) {
      return (String)transGrid.getObject(row, col);   
   }
   
   void setVisible(boolean v) {
      transPanelDecor.setVisible(v);
      transPanel.setVisible(v);
   }
   
   int deleteCurrent(String tradeID) {
      int row;
	 
      if ((row = transGrid.getSelectedRow()) < 0) {
         ITrade.displayStatus("Please select a transaction to delete");
         return -1;
      }
	 
      transGrid.removeRow(row, 1);
      return(transGrid.getRowCount());
   }
}
